--    cr2hdr developped by a1ex
--    http://www.magiclantern.fm/forum/index.php?topic=7139.0
--    Plugin developped for Magic Lantern
--    Add "cr2hdr" to export
--
--    Metadata added by akry
--
--    Copyright (C) <2014>  <kichetof kichetof@gmail.com>
--
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License along
--    with this program; if not, write to the Free Software Foundation, Inc.,
--    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
--
--    cr2hdr (mac) & cr2hdr.exe (win) compiled on mac with nightly builds on march 16 2014.
--    dcraw.exe (june 18 2013)
--    exiftool.exe (july 13 2013)

-- ------------------------------------------------
-- ------------------------------------------------
--
-- Import LR API
--
--
local LrApplication = import 'LrApplication'
local LrBinding = import 'LrBinding'
local LrColor = import 'LrColor'
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrExportSession = import 'LrExportSession'
local LrFileUtils = import 'LrFileUtils'
local LrFunctionContext = import 'LrFunctionContext'
local LrHttp = import 'LrHttp'
local LrLogger = import 'LrLogger'
local LrPathUtils = import 'LrPathUtils'
local LrProgressScope = import 'LrProgressScope'
local LrTasks = import 'LrTasks'
local LrView = import "LrView"
--
--
-- end Import LR API
--
-- ------------------------------------------------
-- ------------------------------------------------

-- ------------------------------------------------
-- ------------------------------------------------
--
-- Custom functions declarations
--
--

-- Check for Mac dependencies
function checkDependencies()
	return "exec which dcraw exiftool"
end

-- Fill empty table for summary message
function checkTab (tab, err)
	local msg = err ~= 1 and LOC "$$$/ML/Publish/Finish/NoImage=No image" or LOC "$$$/ML/Publish/Finish/NoError=No error"
	return #tab == 0 and table.insert(tab, { ["title"] = msg, ["value"] = "" }) or tab
end

-- Update optional arguments when --fast is selected
function updateArguments(propertyTable)
	if propertyTable.shortcut_fast then
		propertyTable.interp_method = 1
		propertyTable.chroma_smooth_method = 0
		propertyTable.use_fullres = 0
		propertyTable.use_alias_map = 0
		propertyTable.use_stripe_fix = 0
	else
		propertyTable.interp_method = 0
		propertyTable.chroma_smooth_method = 2
		propertyTable.use_fullres = 1
		propertyTable.use_alias_map = 1
		propertyTable.use_stripe_fix = 1
	end
end

-- Return command line for cr2hdr
function generateCmd(exportSettings, pluginPath, filePath, output, args)

	local exec = "cr2hdr"
	local gt = WIN_ENV and " ^> " or " > "
	local cmd = WIN_ENV and 'start /D "%s\\bin" /B /WAIT /BELOWNORMAL %s.exe%s "%s"' or 'exec "%s/bin/%s"%s"" "%s"'
	local log = '%s%s"%s"'
	local escape = WIN_ENV and '"%s"' or '%s'
	
	return string.format(escape,
		string.format(log, 
			string.format(cmd,
				pluginPath,
				exec,
				args,
				filePath
			),
			gt,
			output)
	)
	
end

-- remove trailing and leading spaces
function trimLog(s)
	return (s:gsub("^%s*(.-)%s*$", "%1"))
end

-- parse logfile to read info
function parseLogFileToArray(filename)

	local logFileOpen = assert(io.open(filename, 'r'))
	
	local logfile = logFileOpen:read("*all")
	logFileOpen:close() 
	
	info = {}
	info_index = {}
	
	-- parse cr2hdr output for "xxx : yyy" and store keys/values in temp_t
	for k, v in string.gmatch(logfile, '%s*([%w%s-]+)%s*:%s*([%S ]+)%s*') do
	
		k = trimLog(k)

		-- there are duplicating keys in output (for instance, "Dynamic range")
		if info[k] ~= nil then
		
			local t = k
			info_index[t] = info_index[t] + 1
			k = k .. info_index[t]
			
		else
		 
			info_index[k] = 1
		
		end
		
		info[k] = trimLog(v)
		
	end
	
	return info
		
end

-- fill metada
function fillMetadata(photo, meta)

	-- exit function if metadata array is empty
	if meta == nil then
	
		do return end
	
	end
	
	local metaString1
	local metaString2
	
	if meta['ISO difference'] ~= nil then
		metaString1 = meta['ISO difference']:match("(%S+ EV)")
		photo:setPropertyForPlugin (_PLUGIN, 'ISOdifference', metaString1)
	end
	
	if meta['Dynamic range2'] ~= nil then
		metaString1 = meta['Dynamic range2']:match("(%S+ EV)")
		photo:setPropertyForPlugin (_PLUGIN, 'DynamicRange', metaString1)
	end
	
	if meta['Dynamic range'] ~= nil then
		metaString1, metaString2 = meta['Dynamic range']:match("(%S+) %(%+%) (%S+)")
		photo:setPropertyForPlugin (_PLUGIN, 'DynamicRangeOrig1', metaString1 .. ' EV')
		photo:setPropertyForPlugin (_PLUGIN, 'DynamicRangeOrig2', metaString2 .. ' EV')
	end
	
	if meta['Semi-overexposed'] ~= nil then
		metaString1 = meta['Semi-overexposed']:match("(%S+%%)")
		photo:setPropertyForPlugin (_PLUGIN, 'SemiOverexposed', metaString1)
	end
	
	if meta['Deep shadows'] ~= nil then
		metaString1 = meta['Deep shadows']:match("(%S+%%)")
		photo:setPropertyForPlugin (_PLUGIN, 'DeepShadows', metaString1)
	end
	
	
	if meta['Noise level'] ~= nil then
		metaString1, metaString2 = meta['Noise level']:match("(%S+) .+, ideally (%S+)")
		photo:setPropertyForPlugin (_PLUGIN, 'NoiseLevel', metaString1)
		photo:setPropertyForPlugin (_PLUGIN, 'NoiseLevelIdeal', metaString2)
	end
	
	if meta['ISO overlap'] ~= nil then
		metaString1 = meta['ISO overlap']:match("(%S+ EV)")
		photo:setPropertyForPlugin (_PLUGIN, 'ISOoverlap', metaString1)
	end

end
--
--
-- end Custom functions declarations
--
-- ------------------------------------------------
-- ------------------------------------------------

-- ------------------------------------------------
-- ------------------------------------------------
--
-- variables
--
--

-- var for binding variable
local bind = LrView.bind

-- var for summary message
local summary = {
	["success"] = {},
	["nodual"] = {},
	['dual'] = {},
	['format'] = {},
	["fail"] = {},
	["errorMsg"] = {}
}

-- var for log
mlLog = LrLogger('MLDualISO')
mlLog:enable("print") -- replace print with logfile to log into a file in ~/Documents on Mac OS, or My Documents on Windows
--mlLog:disable() -- uncomment it to disable log (only on release version)
--
--
-- end Local variables
--
-- ------------------------------------------------
-- ------------------------------------------------

-- ------------------------------------------------
-- ------------------------------------------------
--
-- Start plugin
--
--
exportServiceProvider = {}

exportServiceProvider.canExportVideo = false

exportServiceProvider.exportPresetFields = {
	{ key = 'suffix',           	default = "dualiso" },
	{ key = 'label',            	default = "DualISO" },
	{ key = 'saveOutput',       	default = true },
	{ key = 'addToCollection',  	default = true },
	{ key = 'subfolder',        	default = "" },
	{ key = 'shortcut_fast',		default = false },
	{ key = 'interp_method',		default = 0 },		--0 amaze, 1 mean23
	{ key = 'chroma_smooth_method',	default = 2 },		--2 cs2, 3 cs3, 5 cs5, 0 no-cs
	{ key = 'fix_bad_pixels',		default = 0 },		--1 bad, 2 really bad, 0 no bad
	{ key = 'debug_bad_pixels',		default = false },
	{ key = 'use_fullres', 			default = 1 },		--0 no full, 1 full
	{ key = 'use_alias_map',		default = 1 },		--0 no map, 1 map
	{ key = 'use_stripe_fix',		default = 1 },		--0 no stripe, 1 stripe
	{ key = 'compress',				default = 0 },		--0 no compress, 1 compress, 2 compress lossy
}

exportServiceProvider.hideSections = { "exportLocation", "fileNaming", "fileSettings", "imageSettings", "outputSharpening", "metadata", "watermarking", "video" }

function exportServiceProvider.startDialog(propertyTable)

	propertyTable:addObserver("shortcut_fast", function() updateArguments(propertyTable) end)
	
end

function exportServiceProvider.endDialog( propertyTable )

	propertyTable.LR_tokens = string.format("{{image_name}}-%s", propertyTable.suffix)
	propertyTable.LR_format = 'ORIGINAL'
	propertyTable.LR_export_colorSpace = "AdobeRGB"

end

function exportServiceProvider.processRenderedPhotos(functionContext, exportContext)

	functionContext:addCleanupHandler(
		function(status, message)
	        if not status then
	        
	              LrDialogs.message("~ There has been an error ~", string.format("Please report the problem to Magic Lantern forum\n\n%s", message), "critical")
	        
	        end
	    end
    )
	
	if MAC_ENV then
		
		local cmd = checkDependencies()
		isDependOk = LrTasks.execute(cmd)

		if isDependOk ~= 0 then
			
			LrDialogs.message(LOC "$$$/ML/Dependencies/ErrorTitle=~ Dependencies error ~", LOC "$$$/ML/Dpendencies/Error=This plugin (cr2hdr) need dcraw and exiftool.^n^nPlease install it before run this plugin")
			do return end
			
		end
	end
	
	local exportSession = exportContext.exportSession
	local exportSettings = exportContext.propertyTable
	local nPhotos = exportSession:countRenditions()
	local progressScope = exportContext:configureProgress {
		title = nPhotos > 1 and LOC("$$$/ML/Publish/Progress=Exporting ^1 photos to cr2hdr", nPhotos) or LOC "$$$/ML/Publish/Progress/One=Exporting one photo to cr2hdr",
	}

	for i, rendition in exportContext:renditions { stopIfCanceled = true } do

		local fileFormat = rendition.photo:getRawMetadata("fileFormat")
		if fileFormat == "RAW" or fileFormat == "DNG" then
			
			progressScope:setPortionComplete((i - 1) / nPhotos)
			
			if not rendition.wasSkipped then
			
				local success, pathOrMessage = rendition:waitForRender()
				progressScope:setPortionComplete((i - 0.5) / nPhotos)
				
				local originalFilePath = rendition.photo.path
				mlLog:debugf("originalFilePath %s", originalFilePath)
				-- original filename to export (exemple img_1234.cr2)
				local originalName = LrPathUtils.leafName(originalFilePath)
				mlLog:debugf("originalName %s", originalName)
				-- original path to reimport
				local originalPath = LrPathUtils.parent(originalFilePath)
				
				if exportSettings.subfolder ~= "" then
				
					originalPath = LrPathUtils.child(originalPath, exportSettings.subfolder)
					local createSubFolder = LrFileUtils.createAllDirectories(originalPath)
					mlLog:debugf("createSubFolder : %s", createSubFolder)
					
				end
				
				mlLog:debugf("originalPath %s", originalPath)
				-- get original collection
				local originalCollection = rendition.photo:getContainedCollections()
				mlLog:debugf("originalCollection %s", originalCollection)
				-- export path & filename (exemple ~/images/dualiso_1234.cr2)
				local filePath = assert(pathOrMessage)
				mlLog:debugf("filePath %s", filePath)
				-- export path & filename for DNG file
				local filePathDNG = LrPathUtils.replaceExtension(filePath, "DNG")
				mlLog:debugf("filePathDNG %s", filePathDNG)
				-- the filename for reimport to LR
				local importFileDNG = LrPathUtils.leafName(filePathDNG)
				mlLog:debugf("importFileDNG %s", importFileDNG)
				-- the filename & path to reimport into LR
				local importDNG = LrPathUtils.child(originalPath, importFileDNG)
				-- possibility to bypass same name; disable to see already treated message
				--local importDNG = LrFileUtils.chooseUniqueFileName(importDNG)
				mlLog:debugf("importDNG %s", importDNG)
				-- export path & filename for txt output command
				local output = LrPathUtils.replaceExtension(importDNG, "txt")
				mlLog:debugf("output %s", output)
				
				if LrFileUtils.exists(importDNG) == false then
				
					if progressScope:isCanceled() then break end
					
					if success then
					
						-- plugin path
						local pluginPath = _PLUGIN.path
						
						local args = string.format('%s%s%s%s%s%s%s%s%s', 
							(exportSettings.shortcut_fast == true) and " --fast" or "",
							(exportSettings.interp_method == 0) and " --amaze-edge" or " --mean23",
							(exportSettings.chroma_smooth_method == 0) and " --no-cs" or string.format(" --cs%dx%d", exportSettings.chroma_smooth_method, exportSettings.chroma_smooth_method),
							(exportSettings.fix_bad_pixels == 0) and " --no-bad-pix" or ((exportSettings.fix_bad_pixels == 1) and " --bad-pix" or " --really-bad-pix"),
							(exportSettings.debug_bad_pixels == true) and " --black-bad-pix" or "",
							(exportSettings.use_fullres == 0) and " --no-fullres" or " --fullres",
							(exportSettings.use_alias_map == 0) and " --no-alias-map" or " --alias-map",
							(exportSettings.use_stripe_fix == 0) and " --no-stripe-fix" or " --stripe-fix",
							(exportSettings.compress == 1) and " --compress" or ((exportSettings.compress == 2) and "--compress-lossy" or ""))
						mlLog:debugf("arguments : %s", args)

						local command = generateCmd(exportSettings, pluginPath, filePath, output, args)
						mlLog:debugf("command : %s", command)
						result = LrTasks.execute(command)
						
						if result == 0 then

							if LrFileUtils.exists(filePathDNG) ~= false then
							
								local success, moveError = LrFileUtils.copy(filePathDNG, importDNG)
								if success then
								
									local catalog = LrApplication:activeCatalog()
									local importDualiso = catalog:withWriteAccessDo('Import from ML DualISO',
										function(context)
											catalog:addPhoto(importDNG, rendition.photo, 'above')
											local photo = catalog:findPhotoByPath(importDNG)
											photo:setRawMetadata('label', exportSettings.label)
											
											local metadata = parseLogFileToArray(output)
											fillMetadata(photo, metadata)
											
											if exportSettings.addToCollection and next(originalCollection) ~= nil then
	
												for k, v in pairs(originalCollection) do
												
													v:addPhotos({ photo })
													
												end
												
											end
											
											if not exportSettings.saveOutput then
												
												LrFileUtils.moveToTrash(output)
											
											end
											
											summary.success[#summary.success+1] = { ["title"] = originalName, ["value"] = "" }
											
										end,
										{timeout = 30}
									)
									
									if importDualiso ~= "executed" then
										
										summary.errorMsg[#summary.errorMsg+1] = { ["title"] = LOC "$$$/ML/Import/Error=Unable to import the dual iso file!^n^nPlease manually synchronise your folder", ["value"] = "" }
										
									end
								else
								
									summary.errorMsg[#summary.errorMsg+1] = { ["title"] = LOC ("$$$/ML/Move/Error=Unable to copy the file^n^nfrom:^n^1^nto:^n^2^nError:^n^3", filePathDNG, importDNG, moveError), ["value"] = "" }

								end
								
							else
								
								summary.nodual[#summary.nodual+1] = { ["title"] = originalName, ["value"] = "" }
								
							end
							
						else
							
							summary.errorMsg[#summary.errorMsg+1] =  { ["title"] = LOC("$$$/ML/Publish/Error=Unable to export ^1 to cr2hdr! Error ^2", originalName, result), ["value"] = "" }
							
							break
						
						-- end if cr2hdr
						end
						
					-- end if success	
					end
					
				else
				
					summary.dual[#summary.dual+1] = { ["title"] = originalName, ["value"] = "" }
					
				end
			-- rendition skip
			end
			
		else

			summary.format[#summary.format+1] = { ["title"] = LrPathUtils.leafName(rendition.photo.path), ["value"] = "" }
		
		-- end if not raw
		end
	
	-- end for
	end
	
	progressScope:done()
		
	-- Show final message
	mlLog:debugf("success: %d - nodual: %d - dual: %d - format: %d - fail: %d - errorMsg: %d",
		#summary.success,
		#summary.nodual,
		#summary.dual,
		#summary.format,
		#summary.fail,
		#summary.errorMsg
		)
		
	local v = LrView.osFactory()
	local totalImg = #summary.success
	local margin = 5
	local width = 600

	LrDialogs.presentModalDialog {
		title = LOC "$$$/ML/Publish/Finish=cr2hdr export finished",
		resizable = "vertically",
		cancelVerb = "< exclude >",
		contents = v:view {
			width = width,
			v:static_text {
				width = width,
				title = totalImg == 0 and LOC "$$$/ML/Publish/NoPhotoExported=No photo has been exported" or totalImg <= 1 and LOC("$$$/ML/Publish/NbPhotoExported=^1 photo has been exported with success", totalImg) or LOC("$$$/ML/Publish/NbPhotosExported=^1 photos have been exported with success", totalImg),
				alignment = "center",
				font = "<system/bold>",
			},
			v:spacer {
				height = 20
			},
			v:tab_view {
				value = "FinishTab1",
				fill_horizontal = 1,
				v:tab_view_item {
					title = LOC("$$$/ML/Publish/Finish/Success=Converted^1", (#summary.success > 0 and " (" .. #summary.success .. ")" or "")),
					identifier = "FinishTab1",
					v:simple_list {
						width = width,
						items = checkTab(summary.success, 0),
					},
				},
				v:tab_view_item {
					title = LOC("$$$/ML/Publish/Finish/noDual=No DualISO^1", (#summary.nodual > 0 and " (" .. #summary.nodual .. ")" or "")),
					identifier = "FinishTab2",
					v:simple_list {
						width = width,
						items = checkTab(summary.nodual, 0),
					},
				},
				v:tab_view_item {
					title = LOC("$$$/ML/Publish/AlreadyConvertTitle=Already treated^1", (#summary.dual > 0 and " (" .. #summary.dual .. ")" or "")),
					identifier = "FinishTab3",
					v:simple_list {
						width = width,
						items = checkTab(summary.dual, 0),
					},
				},
				v:tab_view_item {
					title = LOC("$$$/ML/Publish/Error/NotRawTitle=Wrong format^1", (#summary.format > 0 and " (" .. #summary.format .. ")" or "")),
					identifier = "FinishTab4",
					v:simple_list {
						width = width,
						items = checkTab(summary.format, 0),
					},
				},
				v:tab_view_item {
					title = LOC("$$$/ML/Publish/Finish/Error=Error^1", (#summary.errorMsg > 0 and " (" .. #summary.errorMsg .. ")" or "")),
					identifier = "FinishTab5",
					v:simple_list {
						width = width,
						items = checkTab(summary.errorMsg, 1),
					},
				},
			},
		}
	}
	
-- end function
end

function exportServiceProvider.sectionsForTopOfDialog(f, propertyTable)
	local widthText = 300
	local widthSpacer = 5
	local fillRadio = 0.2
	return {
		{
			title = LOC "$$$/ML/ExportManager/Title=Magic Lantern Dual ISO",
			f:row {
				f:picture {
					value = _PLUGIN:resourceId("logo.png"),
				},
			},
			f:row {
				f:push_button {
					title = LOC "$$$/ML/ExportManager/MLTopic=Magic Lantern Dual ISO thread",
					action = function()
						LrHttp.openUrlInBrowser("http://www.magiclantern.fm/forum/index.php?topic=7139.0")
					end,					
				},
				f:push_button {
					title = LOC "$$$/ML/ExportManager/MLDocTech=Technical doc",
					action = function()
						LrHttp.openUrlInBrowser("http://acoutts.com/a1ex/dual_iso.pdf")
					end,					
				},
				f:spacer {
					height = 20
				},
				
			},
			f:row {
				f:static_text {
					title = LOC "$$$/ML/ExportManager/Description=This plugin allows to convert a Dual ISO picture with cr2hdr (thanks a1ex) from Magic Lantern.\n\nHow it works :\n\n- Export picture to temp folder\n- Add -dualiso suffix to picture\n- Convert picture with cr2hdr\n- Import converted picture into this LR Catalog\n- Add DualISO to the label\n- Add converted picture to the collection of the original",
				},
			},
		},
		{
			title = LOC "$$$/ML/ExportManager/CRTitle=cr2hdr optional arguments",
			visible = false,
			f:tab_view {
				value = "CRTab1",
				fill_horizontal = 1,
				f:tab_view_item {
					title = "Shortcuts",
					identifier = "CRTab1",
					f:row {
						f:checkbox {
							size = "small",
							text_color = LrColor("gray"),
							title = "--fast",
							tooltip = "disable most postprocessing steps (fast, but low quality)\n(--mean23, --no-cs, --no-fullres, --no-alias-map, --no-stripe-fix, --no-bad-pix)",
							value = bind 'shortcut_fast',
							checked_value = true,
							unchecked_value = false,
						},
					},
				},
				f:tab_view_item {
					title = "Interpolation methods",
					identifier = "CRTab2",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--amaze-edge",
							tooltip = "use a temporary demosaic step (AMaZE) followed by edge-directed interpolation (default)",
							value = bind 'interp_method',
							checked_value = 0,
						},
					},
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--mean23",
							tooltip = "average the nearest 2 or 3 pixels of the same color from the Bayer grid (faster)",
							value = bind 'interp_method',
							checked_value = 1,
						},
					},
				},
				f:tab_view_item {
					title = "Chroma smoothing",
					identifier = "CRTab3",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-cs",
							tooltip = "disable chroma smoothing",
							value = bind 'chroma_smooth_method',
							checked_value = 0,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs2x2",
							tooltip = "apply 2x2 chroma smoothing in noisy and aliased areas (default)",
							value = bind 'chroma_smooth_method',
							checked_value = 2,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs3x3",
							tooltip = "apply 3x3 chroma smoothing in noisy and aliased areas",
							value = bind 'chroma_smooth_method',
							checked_value = 3,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs5x5",
							tooltip = "apply 5x5 chroma smoothing in noisy and aliased areas",
							value = bind 'chroma_smooth_method',
							checked_value = 5,
						},
					},
				},
				f:tab_view_item {
					title = "Bad pixel handling",
					identifier = "CRTab4",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-bad-pix",
							tooltip = "disable bad pixel fixing (try it if you shoot stars)",
							value = bind 'fix_bad_pixels',
							checked_value = 3,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--bad-pix",
							tooltip = "",
							value = bind 'fix_bad_pixels',
							checked_value = 1,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--really-bad-pix",
							tooltip = "aggressive bad pixel fix, at the expense of detail and aliasing",
							value = bind 'fix_bad_pixels',
							checked_value = 2,
						},
					},
					f:spacer {},
					f:row {
						f:checkbox {
							size = "small",
							text_color = LrColor("gray"),
							title = "--black-bad-pix",
							tooltip = "mark all bad pixels as black (for troubleshooting)",
							value = bind 'debug_bad_pixels',
							checked_value = true,
							unchecked_value = false,
						},
					},
				},
				f:tab_view_item {
					title = "Other postprocessing steps",
					identifier = "CRTab5",
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-fullres",
							tooltip = "disable full-resolution blending",
							value = bind 'use_fullres',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--fullres",
							tooltip = "",
							value = bind 'use_fullres',
							checked_value = 1,
						},
					},
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-alias-map",
							tooltip = "disable alias map, used to fix aliasing in deep shadows",
							value = bind 'use_alias_map',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--alias-map",
							tooltip = "",
							value = bind 'use_alias_map',
							checked_value = 1,
						},
					},
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-stripe-fix",
							tooltip = "disable horizontal stripe fix",
							value = bind 'use_stripe_fix',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--stripe-fix",
							tooltip = "",
							value = bind 'use_stripe_fix',
							checked_value = 1,
						},
					},
				},
				f:tab_view_item {
					title = "DNG compression",
					identifier = "CRTab6",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-compress",
							tooltip = "Lossless DNG compression",
							value = bind 'compress',
							checked_value = 0,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--compress",
							tooltip = "Lossless DNG compression",
							value = bind 'compress',
							checked_value = 1,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--compress-lossy",
							tooltip = "Lossy DNG compression (be careful, may destroy shadow detail)",
							value = bind 'compress',
							checked_value = 2,
						},
					},
				},
			},
		},
		{
			title = LOC "$$$/ML/ExportManager/ASTitle=Advanced settings",
			visible = false,
			f:row {
				f:static_text {
					alignment = "right",
					width = widthText,
					title = LOC "$$$/ML/ExportManager/ASSuffix=Suffix for output dual iso file :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:static_text {
					size = "small",
					text_color = LrColor("gray"),
					title = "IMG_1234-",
				},
				f:edit_field {
					value = bind 'suffix',
					width = 60,
					
				},
				f:static_text {
					size = "small",
					text_color = LrColor("gray"),
					title = ".dng",
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASLabel=Define the label for Dual ISO file :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:edit_field {
					value = bind 'label',
					width = 100,
				},
			},
			f:row {

				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASSubfolder=Add the converted file to this subfolder :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:edit_field {
					value = bind 'subfolder',
					width = 100,
					placeholder_string = LOC "$$$/ML/ExportManager/ASSubfolderPL=Parent",
				},
				f:static_text {
					text_color = LrColor("gray"),
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASSubfolderInfo=leave blank to reimport into the same folder",
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASCollection=Add Dual ISO file to collection ?",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:checkbox {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASCollectionInfo=Uncheck to not add to the original picture collection",
					value = bind 'addToCollection',
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASLog=Keep log file after conversion ?",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:checkbox {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASLogInfo=Uncheck to remove",
					value = bind 'saveOutput',
					checked_value = true,
					unchecked_value = false,
				},
			},
		},
	}
end

return exportServiceProvider
--
--
-- end Plugin
--
-- ------------------------------------------------
-- ------------------------------------------------